import subprocess
from py_output_compare.user_input_case import InputCase
from py_output_compare.find_file import find_first_file


def get_run_output_by_search_file_name(
    filename, input_data=[InputCase("")], timeout_setting=6
):
    """_summary_

    Args:
        filename (_type_): get file name instead of path, it will try to find by itself
        input_data (_type_):
        timeout_setting (int, optional):. Defaults to 6.
    """
    path = find_first_file(filename)
    return get_run_output_by_path(path, input_data, timeout_setting)


def get_run_output_by_path(file_path, input_data=[InputCase("")], timeout_setting=6):
    output_lines = ""
    try:
        refactor_input = "\n".join((map(str, input_data)))
        result = subprocess.run(
            ["python", file_path],
            input=refactor_input,
            capture_output=True,
            text=True,
            timeout=timeout_setting,
            encoding="utf-8",
        )
        output_lines = result.stdout
        output_lines += result.stderr

        if len(output_lines) == 0:
            raise Exception("your file give no output")

        return output_lines
    except EOFError:
        output_lines += "🔚 End of file Error"
        return output_lines
    except TypeError:
        return "TypeError"
    except subprocess.TimeoutExpired:
        output_lines += "💀 Timed out!!!"
        return output_lines
    except Exception as e:
        output_lines += f"😲 Encountered an exception: {str(e)}"
        return output_lines


def main():
    print(get_run_output_by_search_file_name("print_test.py"))


if __name__ == "__main__":
    main()



from py_output_compare.normalize_file_output import (
    remove_space_tab_newline,
    replace_space_tab_newline,
)


def spot_char_diff(expect_input, got_input):
    max_len = max(len(expect_input), len(got_input))
    result = []

    result.append("diff>   👉[")
    for i in range(max_len):
        char1 = expect_input[i] if i < len(expect_input) else ""
        char2 = got_input[i] if i < len(got_input) else ""
        if char1 == char2:
            result.append(".")
        else:
            result.append("#")
    result.append("]")

    diff_result = "".join(result)
    return diff_result


def highlight_diff(expect_input, got_input):

    if expect_input == got_input:
        return "no different found"

    norm_expect_input = remove_space_tab_newline(expect_input)
    norm_got_input = remove_space_tab_newline(got_input)

    final_word = []
    final_word.append("-" * 80)

    file_have_error_in_space = norm_expect_input == norm_got_input

    if file_have_error_in_space:
        final_word.append("👾: space/newline error! ⚪:space 😶:tab 🌍:newline")
        norm_expect_input = replace_space_tab_newline(expect_input)
        norm_got_input = replace_space_tab_newline(got_input)

    diff_result = spot_char_diff(norm_expect_input, norm_got_input)

    # print compare normalize text
    final_word.append(diff_result)
    final_word.append(f"📩expect> [{norm_expect_input}]")
    final_word.append(f"🧒your's> [{norm_got_input}]")
    final_word.append(diff_result)

    # print real output
    final_word.append("-" * 80)
    final_word.append("💡 expect output: ")
    final_word.append(expect_input)
    final_word.append("-" * 80)
    final_word.append("📃 your output is:")
    final_word.append(got_input)
    final_word.append("~" * 80)
    final_output = "\n".join(final_word)
    return final_output


def main():
    # print(highlight_diff("hello", "hello"))
    print(highlight_diff("he llo", "hello"))
    print(highlight_diff("hello\n", "hello"))
    print(highlight_diff("\nhello", "hello"))
    print(highlight_diff("hell o", "h ell o"))
    # print(highlight_diff("hedsf sdfllo", "hsdf ello"))


if __name__ == "__main__":
    main()



import os


def find_files(filename, base_dir="./", case_sensitive=True, find_all=True):
    count = 0
    filename_lower = filename.lower() if not case_sensitive else None
    for root, _, files in os.walk(base_dir):
        for file in files:
            if (case_sensitive and file == filename) or (
                not case_sensitive and file.lower() == filename_lower
            ):
                full_path = os.path.join(root, file)
                count += 1
                yield full_path
                if not find_all:
                    print(f"{count} file found.")
                    return
    print("*" * 105)
    print(f"{count} file{'s' if count != 1 else ''} found.")
    print("*" * 105)


def find_first_file(filename, base_dir="./", case_sensitive=True):
    filename_lower = filename.lower() if not case_sensitive else None
    for root, _, files in os.walk(base_dir):
        for file in files:
            if (case_sensitive and file == filename) or (
                not case_sensitive and file.lower() == filename_lower
            ):
                full_path = os.path.join(root, file)
                print("File found.")
                return full_path
    print("No file found.")
    return None


def find_first_file_contain_id(filename, folder_id):
    file_list = find_files(filename)
    for file in file_list:
        if folder_id in file:
            return file
    print("no file found")
    return None


def main():
    for file in find_files("test_class.py"):
        print(file)

    print(find_first_file_contain_id("test_class.py", "py_output_compare"))
    print(find_first_file("test_class.py"))


if __name__ == "__main__":
    main()



def normalize_output(output):
    return output.strip().lower().replace("\n", "").replace(" ", "").replace("\t", "")


def remove_space_tab_newline(output):
    return output.replace(" ", "").replace("\t", "").replace("\n", "")


def replace_space_tab_newline(output):
    return output.replace(" ", "⚪").replace("\t", "😶").replace("\n", "🌍")



import os
from py_output_compare import find_files, find_first_file, find_first_file_contain_id


def print_path():
    print(find_first_file_contain_id("find_test.txt", "test"))


def test_find_files_all_files():
    filename = "find_test.txt"
    base_dir = r"tests\test_lab_script"
    case_sensitive = True
    find_all = True
    expected_files = [
        os.path.join(base_dir, filename),
    ]

    assert (
        list(find_files(filename, base_dir, case_sensitive, find_all)) == expected_files
    )


def test_find_files_single_file():
    filename = "find_test.txt"
    base_dir = r"tests\test_lab_script"
    case_sensitive = True
    find_all = False
    expected_file = os.path.join(base_dir, filename)
    assert (
        next(find_files(filename, base_dir, case_sensitive, find_all)) == expected_file
    )


def test_find_first_file():
    filename = "find_test.txt"
    base_dir = r"tests\test_lab_script"
    case_sensitive = True
    expected_file = os.path.join(base_dir, filename)
    assert find_first_file(filename, base_dir, case_sensitive) == expected_file


def test_find_first_file_not_found():
    filename = "not_found.txt"
    base_dir = "./"
    case_sensitive = True
    assert find_first_file(filename, base_dir, case_sensitive) is None


def test_find_first_file_contain_id():
    filename = "find_test.txt"
    folder_id = "tests"
    expected_file = os.path.join("./", r"tests\test_lab_script", filename)
    assert find_first_file_contain_id(filename, folder_id) == expected_file


def test_find_first_file_contain_id_not_found():
    filename = "find_test.txt"
    folder_id = "not_found"
    assert find_first_file_contain_id(filename, folder_id) is None



from py_output_compare.user_input_case import InputCase
from py_output_compare.highlight import highlight_diff
from py_output_compare.normalize_file_output import normalize_output
from py_output_compare.find_file import find_first_file
from py_output_compare.get_file_run_output import get_run_output_by_path


def get_compare_output_by_search_file_name(
    filename_1,
    filename_2,
    user_input_list=[InputCase("")],
    do_normalize_output=False,
    timeout=6,
):
    filepath_1 = find_first_file(filename_1)
    filepath_2 = find_first_file(filename_2)
    return get_compare_output_by_path(
        filepath_1,
        filepath_2,
        user_input_list,
        do_normalize_output,
        timeout,
    )


def get_compare_output_by_path(
    file_path_1,
    file_path_2,
    user_input_list=[InputCase("")],
    do_normalize_output=False,
    timeout=6,
):
    result = []
    score = []
    result.append("=" * 80)

    for user_input in user_input_list:
        file_output_1 = get_run_output_by_path(
            file_path_1, user_input.case_input, timeout
        )
        file_output_2 = get_run_output_by_path(
            file_path_2, user_input.case_input, timeout
        )

        if do_normalize_output:
            file_output_2 = normalize_output(file_output_2)
            file_output_1 = normalize_output(file_output_1)

        if file_output_2 == file_output_1:
            result.append(f"✅: {user_input.case_name} pass!")
            score.append("🟢")

        else:
            result.append("~" * 80)
            result.append(f"❌: {user_input.case_name} fail!")
            score.append("🔴")
            result.append(highlight_diff(file_output_2, file_output_1))

    final_score = "".join(score)
    result.append("=" * 80)

    result.append(f"got {final_score} scores")
    result.append("=" * 80)

    final_compare_result = "\n".join(result)
    return final_compare_result


def get_score_by_path(
    file_path_1,
    file_path_2,
    user_input_list=[InputCase("")],
    do_normalize_output=False,
    timeout=6,
):
    result = []
    score_num = 0
    score = []

    for user_input in user_input_list:
        file_output_1 = get_run_output_by_path(
            file_path_1, user_input.case_input, timeout
        )
        file_output_2 = get_run_output_by_path(
            file_path_2, user_input.case_input, timeout
        )

        if do_normalize_output:
            file_output_1 = normalize_output(file_output_1)
            file_output_2 = normalize_output(file_output_2)

        if file_output_2 == file_output_1:
            result.append(f"✅: {user_input.case_name} pass!")
            score.append("🟢")
            score_num += 1

        else:
            result.append("~" * 80)
            result.append(f"❌: {user_input.case_name} fail!")
            score.append("🔴")
            result.append(highlight_diff(file_output_2, file_output_1))

    final_emoji_score = "".join(score)
    return score_num, final_emoji_score


def get_score_by_search_file_name(
    filename_1,
    filename_2,
    user_input_list=[InputCase("")],
    do_normalize_output=False,
    timeout=6,
):
    filepath_1 = find_first_file(filename_1)
    filepath_2 = find_first_file(filename_2)
    return get_score_by_path(
        filepath_1,
        filepath_2,
        user_input_list,
        do_normalize_output,
        timeout,
    )


def main():
    student_file = find_first_file("bad.py")
    student_file_good = find_first_file("good.py")
    teacher_file = find_first_file("teacher_file.py")

    case1_input = [8.2, 1.8]
    case_input_int = [8.2, 999, 9334]

    test_cases = [
        InputCase(case1_input),
        InputCase(case_input_int),
    ]

    # print(student_file)
    # print(student_file_good)
    # print(teacher_file)

    # fail_student = get_compare_output_by_path(student_file, teacher_file, test_cases)
    # pass_student = get_compare_output_by_path(
    #     student_file_good, teacher_file, test_cases
    # )
    # no_test_input = get_compare_output_by_path(student_file_good, teacher_file)

    # print(fail_student)
    # print(pass_student)
    # print(no_test_input)

    # print(get_compare_output_by__search_file_name("good.py", "bad.py"))
    # print(get_compare_output_by__search_file_name("print_test.py", "print_test2.py"))
    print(get_score_by_search_file_name("good.py", "good.py"))
    print(get_score_by_search_file_name("good.py", "bad.py"))


if __name__ == "__main__":

    main()



from .user_input_case import *

from .hello import *
from .highlight import *
from .normalize_file_output import *

from .get_file_run_output import *
from .compare_file import *
from .find_file import *

from .file_format_helper import *



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    description_readme = f.read()

with open("version.txt", "r") as version_file:
    version = version_file.read().strip()

setup(
    name="py_output_compare",
    version=version,
    description="a script that help compare output of 2 python script, I apply this to help grading student code compare to teacher",
    long_description=description_readme,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    author="meangpu",
    license="MIT",
    url="https://github.com/meangpu/py_output_compare",
    entry_points={"console_scripts": ["mep = py_output_compare.hello:print_hello"]},
)



from py_output_compare import (
    normalize_output,
    remove_space_tab_newline,
)


def test_normalize_output():
    assert normalize_output(" Hello World \n") == "helloworld"
    assert normalize_output("  \tHello\nWorld\t ") == "helloworld"
    assert normalize_output("123 \t 456\n") == "123456"
    assert normalize_output("") == ""
    assert normalize_output("  ") == ""


def test_normalize_output_no_lower():
    assert remove_space_tab_newline(" Hello World \n") == "HelloWorld"
    assert remove_space_tab_newline("  \tHello\nWorld\t ") == "HelloWorld"
    assert remove_space_tab_newline("123 \t 456\n") == "123456"
    assert remove_space_tab_newline("") == ""
    assert remove_space_tab_newline("  ") == ""



import re
from pathlib import Path


def remove_prefix_from_test(filename):
    """my test lab in formate of 1_1_lab_name
    Returns:
        _type_: file name with no 1_1_ like only lab_name
    """
    return re.sub(r"^\d+_\d+_", "", filename)


def get_lab_name_from_test_file_name(file_path_by_name):
    """Extract the lab name from the file name and include the file extension.
    Args:
        file_path_by_name (str): File path or filename to process.
    Returns:
        str: Lab name extracted from the filename with the file extension.
    """
    full_test_name = Path(file_path_by_name).resolve().stem  # _1_1_adder.py
    final_name = remove_prefix_from_test(full_test_name)
    file_extension = Path(file_path_by_name).suffix  # .py
    return f"{final_name}{file_extension}"
